using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.FxCop.Sdk.Introspection;
using Microsoft.Cci;
using System.Web;
using Microsoft.FxCop.Sdk;
using System.IO;

namespace CustomStaticCodeAnalysisRules
{
    public class ResponseRedirectCallsAreRelative : BaseIntrospectionRule
    {
        public ResponseRedirectCallsAreRelative()
            : base("ResponseRedirectCallsAreRelative", "CustomStaticCodeAnalysisRules.Rules.xml", 
            typeof(ResponseRedirectCallsAreRelative).Assembly)
        {

        }



        private void CheckResponseRedirectArgument(
			Instruction nextInstruction)
        {
            string targetUrl = nextInstruction.Value as string;

            if (targetUrl != null)
            {
                if (targetUrl != null)
                {
                    if (targetUrl.StartsWith("/") == true)
                    {
                        // this is bad
                        Resolution resolution = base.GetResolution(targetUrl);
                        Problem problem = new Problem(resolution, targetUrl);
                        base.Problems.Add(problem);
                    }
                }
            }
        }
        public override ProblemCollection Check(Member member)
        {
            // string responseRedirect = "Response.Redirect";

            Method method = member as Method;

            if (method == null)
            {
                return null;
            }

            // File.AppendAllText(@"c:\rule.log", "Method: " + method.FullName + Environment.NewLine);

            Method target = null;

            int index = 0;
            foreach (Instruction inst in method.Instructions)
            {
                if (inst.OpCode == OpCode.Call || 
					inst.OpCode == OpCode.Callvirt)
                {
                    target = inst.Value as Method;

                    if (target.Name.Name.Contains("Redirect") == true)
                    {
                        // File.AppendAllText(@"c:\rule.log", "Method Name: " + target.FullName + Environment.NewLine);

                        if (index > 0 && 
                            method.Instructions[index - 1].OpCode == OpCode.Ldstr)
                        {
                            Instruction nextInstruction = method.Instructions[index - 1];

                            CheckResponseRedirectArgument(nextInstruction);
                        }
                        else if (index > 1 &&
                            method.Instructions[index - 2].OpCode == OpCode.Ldstr)
                        {
                            Instruction nextInstruction = method.Instructions[index - 2];

                            CheckResponseRedirectArgument(nextInstruction);
                        }
                    }
                }

                index++;
            }

            /*
            if (method.FullName.Contains(responseRedirect) == true)
            {
                if (method.Parameters.Length > 0 &&
                    method.Parameters[0].Type == SystemTypes.String)
                {
                    string targetUrl = method.Parameters[0].Name.Name;

                    if (targetUrl != null)
                    {
                        if (targetUrl.StartsWith("~") == true)
                        {
                            // this is fine
                        }
                        else if (targetUrl.StartsWith("/") == true)
                        {
                            // this is bad
                            Resolution resolution = base.GetResolution(targetUrl);
                            Problem problem = new Problem(resolution, targetUrl);
                            base.Problems.Add(problem);
                        }
                    }
                }
            }
             * */

            return base.Problems;
        }

        
    }
}
